#ifndef BmlLc1176Configuration_HH
#define BmlLc1176Configuration_HH

#include <iostream>
#include <fstream>

#include "RcdHeader.hh"
#include "RcdUserRW.hh"

#include "SubAccessor.hh"
#include "SubInserter.hh"

#include "DaqRunType.hh"

#include "BmlLc1176ConfigurationData.hh"


class BmlLc1176Configuration : public RcdUserRW {

public:
  BmlLc1176Configuration() {
  }

  virtual ~BmlLc1176Configuration() {
  }

  virtual bool record(RcdRecord &r) {
    if(doPrint(r.recordType())) {
      std::cout << "BmlLc1176Configuration::record()" << std::endl;
      r.RcdHeader::print(std::cout," ") << std::endl;
    }
    
    // Check record type
    switch (r.recordType()) {
      
      // Run start 
    case RcdHeader::runStart: {

      // Access the DaqRunStart
      SubAccessor accessor(r);
      std::vector<const DaqRunStart*>
        v(accessor.extract<DaqRunStart>());
      assert(v.size()==1);
      if(doPrint(r.recordType(),1)) v[0]->print(std::cout," ") << std::endl;

      _runType=v[0]->runType();

      // Disable?
      break;
    }
      
      // Configuration start 
    case RcdHeader::configurationStart: {
      
      // Handle the ones which need work
      if(_runType.majorType()==DaqRunType::bml ||
	 _runType.majorType()==DaqRunType::beam ||

	 _runType.type()==DaqRunType::emcBeam ||
	 _runType.type()==DaqRunType::emcBeamHoldScan ||

	 _runType.type()==DaqRunType::ahcBeam ||
	 _runType.type()==DaqRunType::ahcBeamHoldScan ||
	 _runType.type()==DaqRunType::ahcBeamStage ||
	 _runType.type()==DaqRunType::ahcBeamStageScan) {


	SubInserter inserter(r);
	BmlLc1176ConfigurationData
	  *b(inserter.insert<BmlLc1176ConfigurationData>(true));
	b->label(1);

	//const unsigned char v(_runType.version());

	switch(_runType.type()) {

	case DaqRunType::bmlTest: {
	  b->csr(0x00008600);
	  break;
	}
      
	case DaqRunType::bmlNoise: {
	  b->csr(0x00008600);
	  break;
	}
      
	case DaqRunType::bmlInternalTest: {
	  b->csr(0x000040ff);
	  break;
	}
      
	case DaqRunType::emcBeam:
	case DaqRunType::emcBeamHoldScan:

	case DaqRunType::ahcBeam:
	case DaqRunType::ahcBeamHoldScan:
	case DaqRunType::ahcBeamStage:
	case DaqRunType::ahcBeamStageScan:

	case DaqRunType::beamTest:
	case DaqRunType::beamNoise:
	case DaqRunType::beamData:
	case DaqRunType::beamHoldScan:
	case DaqRunType::beamStage:
	case DaqRunType::beamStageScan: {
	  b->csr(0x00008600);
	  break;
	}
	  
	default: {
	  break;
	}
	};
	
	if(doPrint(r.recordType(),1)) b->print(std::cout," ") << std::endl;
      }

      break;
    }
      
    default: {
      break;
    }
    };
    
    return true;
  }


private:
  DaqRunType _runType;
};

#endif
