#ifndef EmcStageReadout_HH
#define EmcStageReadout_HH

#include <iostream>
#include <fstream>

// dual/inc/utl
#include "UtlPack.hh"

// dual/inc/rcd
#include "RcdUserBase.hh"
#include "RcdHeader.hh"

// dual/inc/emc
#include "EmcStageStartupData.hh"

// dual/inc/sub
#include "SubInserter.hh"

// dual/inc/skt
#include "DuplexSocket.hh"


class EmcStageReadout : public RcdUserBase {

public:
  EmcStageReadout(std::string ipAddress="134.158.90.89",
		  unsigned port=1200, unsigned tries=10) :
    RcdUserBase(), _socket(ipAddress.c_str(),port,tries) {
    
    // Construct "magic numbers" to send
    _sendData[0]=0x4c554150; // "PAUL" endian-swapped
    _sendData[1]=0x30313030; // "0010" endian-swapped = length of 10 words
    _sendLength=8;
  }

  virtual ~EmcStageReadout() {
  }

  void bad() {
    _sendData[0]=0x12345678;
    _sendData[1]=0x87654321;
  }

  void good() {
    _sendData[0]=0x4c554150; // "PAUL" endian-swapped
    _sendData[1]=0x30313030; // "0010" endian-swapped = length of 10 words
  }


  bool record(RcdRecord &r) {

    // Check record type
    if(r.recordType()==RcdHeader::startUp  ||
       r.recordType()==RcdHeader::runStart ||
       r.recordType()==RcdHeader::runEnd   ||
       r.recordType()==RcdHeader::runStop) {
      
      if(!_socket.send((const char*)_sendData,_sendLength)) {
	std::cerr << "EmcStageReadout::record()  Error writing to socket;"
		  << " number of bytes written < 8" << std::endl;
	perror(0);
	return false;
      }

      SubInserter inserter(r);
      EmcStageStartupData *d(inserter.insert<EmcStageStartupData>());
	
      int n(-1);
      n=_socket.recv((char*)d,4);
      if(n!=4) {
	std::cerr << "Error reading from socket; number of bytes written = "
		  << n << " < 4" << std::endl;
	perror(0);
	return false;
      }

      if(d->headerError()) {
	std::cerr << "Error in header from socket" << std::endl;
	return false;
      }

      n=_socket.recv(((char*)d)+4,sizeof(EmcStageStartupData)-4);
      if(n!=sizeof(EmcStageStartupData)-4) {
	std::cerr << "Error reading from socket; number of bytes written = "
		  << n << " < " << sizeof(EmcStageStartupData)-4
		  << std::endl;
	perror(0);
	return false;
      }
      
      if(_printLevel>1) d->print(std::cout);
    }

    return true;
  }
    
private:
  DuplexSocket _socket;
  unsigned _sendLength;
  unsigned _sendData[2];
};

#endif
